@@ -68,10 +68,37 @@ cherokee_buffer_free (cherokee_buffer_t *buf)
 }
 
 
+ret_t 
+cherokee_buffer_init (cherokee_buffer_t *buf)
+{
+	buf->buf  = NULL;
+	buf->len  = 0;
+	buf->size = 0;
+
+	return ret_ok;
+}
+
+ret_t 
+cherokee_buffer_mrproper (cherokee_buffer_t *buf)
+{
+	if (buf->buf) {
+		free (buf->buf);
+	}
+
+	return cherokee_buffer_init (buf);
+}
+
 ret_t
 cherokee_buffer_clean (cherokee_buffer_t *buf)
 {
-	return cherokee_buffer_make_empty (buf);
+	if ((buf->buf != NULL) &&
+	    (buf->len > 0))
+	{
+		buf->buf[0] = '\0';
+	}
+
+	buf->len = 0;	
+	return ret_ok;
 }
 
 
@@ -169,20 +196,6 @@ cherokee_buffer_is_endding (cherokee_buffer_t *buf, char c)
 }
 
 
-ret_t
-cherokee_buffer_make_empty (cherokee_buffer_t *buf)
-{
-	if ((buf->buf != NULL) &&
-	    (buf->len > 0))
-	{
-		buf->buf[0] = '\0';
-	}
-
-	buf->len = 0;	
-	return ret_ok;
-}
-
-
 ret_t
 cherokee_buffer_move_to_begin (cherokee_buffer_t *buf, int pos)
 {
