@@ -53,6 +53,9 @@ ret_t cherokee_buffer_new                (cherokee_buffer_t **buf);
 ret_t cherokee_buffer_free               (cherokee_buffer_t  *buf);
 ret_t cherokee_buffer_clean              (cherokee_buffer_t  *buf);
 
+ret_t cherokee_buffer_init               (cherokee_buffer_t  *buf);
+ret_t cherokee_buffer_mrproper           (cherokee_buffer_t  *buf);
+
 ret_t cherokee_buffer_add                (cherokee_buffer_t  *buf, char *txt, size_t size);
 ret_t cherokee_buffer_add_va             (cherokee_buffer_t  *buf, char *format, ...);
 ret_t cherokee_buffer_prepend            (cherokee_buffer_t  *buf, char *txt, size_t size);
@@ -73,7 +76,6 @@ ret_t cherokee_buffer_remove_string      (cherokee_buffer_t  *buf, char *string,
 ret_t cherokee_buffer_replace_string     (cherokee_buffer_t  *buf, char *subs, int subs_len, char *repl, int repl_len);
 
 ret_t cherokee_buffer_ensure_size        (cherokee_buffer_t  *buf, int size);
-ret_t cherokee_buffer_make_empty         (cherokee_buffer_t  *buf);
 
 int   cherokee_buffer_is_empty           (cherokee_buffer_t  *buf);
 int   cherokee_buffer_is_endding         (cherokee_buffer_t  *buf, char c);
