@@ -120,14 +120,18 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 
 	cherokee_buffer_new (&n->local_directory);
 	cherokee_buffer_new (&n->web_directory);
+	cherokee_buffer_new (&n->effective_directory);
 	cherokee_buffer_new (&n->userdir);
 	cherokee_buffer_new (&n->request);
+	cherokee_buffer_new (&n->pathinfo);
 	cherokee_buffer_new (&n->redirect);
 	cherokee_buffer_new (&n->host);
 	cherokee_buffer_new (&n->query_string);
 	cherokee_buffer_new (&n->user);
 	cherokee_buffer_new (&n->passwd);
 
+	cherokee_buffer_init (&n->request_original);
+
 	cherokee_buffer_escape_new (&n->request_escape);
 	cherokee_buffer_escape_set_ref (n->request_escape, n->request);
 
@@ -167,7 +171,9 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 	
 	cherokee_buffer_escape_free (cnt->request_escape);
 	cherokee_buffer_free (cnt->request);
+	cherokee_buffer_mrproper (&cnt->request_original);
 
+	cherokee_buffer_free (cnt->pathinfo);
 	cherokee_buffer_free (cnt->user);
 	cherokee_buffer_free (cnt->passwd);
 	cherokee_buffer_free (cnt->buffer);
@@ -177,10 +183,11 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 
 	cherokee_buffer_free (cnt->local_directory);
 	cherokee_buffer_free (cnt->web_directory);
+	cherokee_buffer_free (cnt->effective_directory);
 	cherokee_buffer_free (cnt->userdir);
 	cherokee_buffer_free (cnt->redirect);
 	cherokee_buffer_free (cnt->host);
-	
+
 	if (cnt->arguments != NULL) {
 		cherokee_table_free2 (cnt->arguments, free);
 		cnt->arguments = NULL;
@@ -239,11 +246,14 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 	cherokee_buffer_clean (cnt->request);
 	cherokee_buffer_escape_clean (cnt->request_escape);
 	cherokee_buffer_escape_set_ref (cnt->request_escape, cnt->request);
+	cherokee_buffer_mrproper (&cnt->request_original);
 
+	cherokee_buffer_clean (cnt->pathinfo);
 	cherokee_buffer_clean (cnt->user);
 	cherokee_buffer_clean (cnt->passwd);
 	cherokee_buffer_clean (cnt->local_directory);
 	cherokee_buffer_clean (cnt->web_directory);
+	cherokee_buffer_clean (cnt->effective_directory);
 	cherokee_buffer_clean (cnt->userdir);
 	cherokee_buffer_clean (cnt->redirect);
 	cherokee_buffer_clean (cnt->host);
@@ -296,7 +306,7 @@ cherokee_connection_mrproper (cherokee_connection_t *cnt)
 	/* It is not a keep-alive connection, so we shouldn't
 	 * keep any previous header
 	 */
-	cherokee_buffer_make_empty (cnt->incoming_header);
+	cherokee_buffer_clean (cnt->incoming_header);
 
 	return ret_ok;
 }
@@ -379,7 +389,7 @@ process_handler_complex_headers (cherokee_connection_t *cnt)
 	/* Location: 
 	 */
 	if (cherokee_header_get_known (header, header_location, &info, &info_len) == ret_ok) {
-		cherokee_buffer_make_empty (cnt->redirect);
+		cherokee_buffer_clean (cnt->redirect);
 		cherokee_header_copy_known (header, header_location, cnt->redirect);
 		cnt->error_code = http_moved_permanently;
 	}
@@ -464,7 +474,7 @@ build_response_header (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
 {	
 	/* Build the response header
 	 */
-	cherokee_buffer_make_empty (buffer);
+	cherokee_buffer_clean (buffer);
 
 	/* Add protocol string + error_code
 	 */
@@ -664,7 +674,7 @@ cherokee_connection_send_header_and_mmaped (cherokee_connection_t *cnt)
 		cnt->mmaped     += offset;
 		cnt->mmaped_len -= offset;
 
-		cherokee_buffer_make_empty (cnt->buffer);
+		cherokee_buffer_clean (cnt->buffer);
 
 		return ret_eagain;
 	}
@@ -766,7 +776,7 @@ cherokee_connection_send_header (cherokee_connection_t *cnt)
 	/* Drop out the sent info
 	 */
 	if (sent == cnt->buffer->len) {
-		cherokee_buffer_make_empty (cnt->buffer);
+		cherokee_buffer_clean (cnt->buffer);
 		return ret_ok;
 	}
 
@@ -795,7 +805,7 @@ cherokee_connection_send (cherokee_connection_t *cnt)
 	/* Drop out the sent info
 	 */
 	if (sent == cnt->buffer->len) {
-		cherokee_buffer_make_empty (cnt->buffer);
+		cherokee_buffer_clean (cnt->buffer);
 	} else if (sent != 0) {
 		cherokee_buffer_move_to_begin (cnt->buffer, sent);
 	}
@@ -1064,6 +1074,8 @@ cherokee_connection_build_local_directory (cherokee_connection_t *cnt, cherokee_
 	ret_t ret;
 
 	if (entry->document_root->len >= 1) {
+		cherokee_buffer_add_buffer (&cnt->request_original, cnt->request);
+
 		/* Have a special DocumentRoot
 		 */
 		ret = cherokee_buffer_add_buffer (cnt->local_directory, entry->document_root);
@@ -1224,7 +1236,7 @@ post_init (cherokee_connection_t *cnt)
 
 	/* Clear the buffer to receive the post data
 	 */
-	cherokee_buffer_make_empty (cnt->post);
+	cherokee_buffer_clean (cnt->post);
 	
 	return ret_ok;
 }
@@ -1555,8 +1567,10 @@ cherokee_connection_create_handler (cherokee_connection_t *cnt, cherokee_handler
 	/* Create and assign a handler object
 	 */
 	ret = (plugin_entry->handler_new_func) ((void **)&cnt->handler, cnt, plugin_entry->properties);
-	if ((ret != ret_ok) || (cnt->handler == NULL)) {
-		cnt->error_code = http_internal_error;
+	if (ret != ret_ok) {
+		if ((cnt->handler == NULL) && (cnt->error_code == http_ok)) {
+			cnt->error_code = http_internal_error;
+		}
 		return ret_error;
 	}
 
