@@ -769,8 +769,8 @@ cherokee_header_foreach (cherokee_header_t *hdr, cherokee_header_foreach_func_t
 			header_name = known_headers_names[i].name;
 			header_len  = known_headers_names[i].len;
 
-			cherokee_buffer_make_empty (name);
-			cherokee_buffer_make_empty (cont);
+			cherokee_buffer_clean (name);
+			cherokee_buffer_clean (cont);
 
 			cherokee_buffer_add (name, header_name, header_len);
 			cherokee_header_copy_known (hdr, i, cont);
@@ -787,8 +787,8 @@ cherokee_header_foreach (cherokee_header_t *hdr, cherokee_header_foreach_func_t
 		header_name = hdr->unknowns[i].header_off + hdr->input_buffer->buf;
 		header_len = strchr (header_name, ':') - header_name;
 
-		cherokee_buffer_make_empty (name);
-		cherokee_buffer_make_empty (cont);
+		cherokee_buffer_clean (name);
+		cherokee_buffer_clean (cont);
 
 		cherokee_buffer_add(name, header_name, header_len);
 		cherokee_buffer_add(cont, 
