@@ -176,7 +176,7 @@ cherokee_logger_ncsa_flush (cherokee_logger_ncsa_t *logger)
 
 	if (logger->accesslog_fd == NULL) {
 		syslog (LOG_INFO, "%s", LOGGER_BUFFER(logger)->buf);
-		return cherokee_buffer_make_empty (LOGGER_BUFFER(logger));
+		return cherokee_buffer_clean (LOGGER_BUFFER(logger));
 	}
 
 
@@ -187,7 +187,7 @@ cherokee_logger_ncsa_flush (cherokee_logger_ncsa_t *logger)
 	}
 
 	if (tmp == LOGGER_BUFFER(logger)->len) {
-		return cherokee_buffer_make_empty (LOGGER_BUFFER(logger));
+		return cherokee_buffer_clean (LOGGER_BUFFER(logger));
 	} 
 
 	return cherokee_buffer_drop_endding (LOGGER_BUFFER(logger), tmp);
@@ -205,7 +205,8 @@ build_log_string (cherokee_logger_ncsa_t *logger, cherokee_connection_t *cnt, ch
 	struct tm         *conn_time;
 	char               ipaddr[CHE_INET_ADDRSTRLEN];
 	cherokee_buffer_t *combined_info = NULL;
-    
+	cherokee_buffer_t *request;
+
 	/* Read the bogonow value from the server
 	 */
 	conn_time = &CONN_THREAD(cnt)->bogo_now_tm;
@@ -256,6 +257,9 @@ build_log_string (cherokee_logger_ncsa_t *logger, cherokee_connection_t *cnt, ch
 		cherokee_buffer_free (useragent);
 	}
 
+	request = cherokee_buffer_is_empty(&cnt->request_original) ? 
+		cnt->request : &cnt->request_original;
+
 	/* Build the log string
 	 */
 	*len = snprintf (tmp, tmp_size,
@@ -272,7 +276,7 @@ build_log_string (cherokee_logger_ncsa_t *logger, cherokee_connection_t *cnt, ch
 			 (int) z/60, 
 			 (int) z%60, 
 			 method,
-			 cnt->request->buf, 
+			 request->buf, 
 			 version, 
 			 cnt->error_code,
 			 (CST_OFFSET) cnt->range_end - cnt->range_start,
